"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

# -*- coding: utf-8 -*-

from werkzeug.contrib.cache import SimpleCache
from flask import request
from utilities import config_manager

cache = SimpleCache()


class CachedMethod(object):

    def __init__(self, timeout=config_manager.get_update_interval()):

        self.timeout = timeout

    def __call__(self, fn):

        def decorator(*args, **kwargs):
            key = (fn.__name__,) + args
            value = cache.get(key)
            if 'clear_cache' in kwargs:
                del(kwargs['clear_cache'])
                value = fn(*args, **kwargs)
                cache.set(key, value, config_manager.get_update_interval())
            elif value is None:
                value = fn(*args, **kwargs)
                cache.set(key, value, config_manager.get_update_interval())
            return value

        return decorator


class CachedUrl(object):

    def __init__(self, timeout=config_manager.get_update_interval()):

        self.timeout = timeout

    def __call__(self, f):

        def decorator(*args, **kwargs):
            response = cache.get(request.path)
            if response is None:
                response = f(*args, **kwargs)
                cache.set(request.path, response, config_manager.get_update_interval())
            return response

        return decorator
